--[[
	I3DLoadingFix
	Specialization to fix the filename of i3d files to allow them load out from an mod!
	
	@author:    Ifko[nator]
	@date:      31.03.2019
	@version:	1.0
]]

I3DLoadingFix = {};
I3DLoadingFix.currentModDirectory = g_currentModDirectory;

local function loadSharedI3DFileFix(self, superFunc, filename, baseDir, callOnCreate, addToPhysics, verbose, asyncCallbackFunction, asyncCallbackObject, asyncCallbackArguments)
    callOnCreate = Utils.getNoNil(callOnCreate, false);
    addToPhysics = Utils.getNoNil(addToPhysics, false);

    local filenameToFix = filename;
    local filename = Utils.getFilename(filename, baseDir);

    --print("filenameToFix = " .. tostring(filenameToFix))
	
	if not fileExists(filename) then
		--print("Can't load '" .. filename .. "'! Try to fix it now!");
		
        filename = Utils.getFilename(filenameToFix, I3DLoadingFix.currentModDirectory); --## here we fix the filename!
        
		--print("Fixed filename '" .. filename .. "'.");
	end;

    local sharedI3D = self.sharedI3DFiles[filename];

    --## always print all loading texts
    verbose = true;

    if asyncCallbackFunction ~= nil then
        if sharedI3D == nil then
            local callbacks = self.sharedI3DFilesPendingCallbacks[filename];

            if callbacks == nil then
                self.sharedI3DFilesPendingCallbacks[filename] = {};
                table.insert(self.sharedI3DFilesPendingCallbacks[filename], {callOnCreate, addToPhysics, asyncCallbackFunction, asyncCallbackObject, asyncCallbackArguments});
                streamI3DFile(filename, "loadSharedI3DFileFinished", self, {filename}, false, false, verbose);
            else
                table.insert(callbacks, {callOnCreate, addToPhysics, asyncCallbackFunction, asyncCallbackObject, asyncCallbackArguments});
            end;
        else
            local id = 0;

            if sharedI3D.nodeId == 0 then
                print("Error: failed to load i3d file '" .. filename .. "'.");
            else
                id = clone(sharedI3D.nodeId, false, callOnCreate, addToPhysics);
            end;

            sharedI3D.refCount = sharedI3D.refCount + 1;

            asyncCallbackFunction(asyncCallbackObject, id, asyncCallbackArguments);
        end;
    else
        if sharedI3D == nil then
            local nodeId = loadI3DFile(filename, false, false, verbose);

            sharedI3D = {nodeId = nodeId, refCount = 0};

            self.sharedI3DFiles[filename] = sharedI3D;
        end;

        local id = 0;

        if sharedI3D.nodeId == 0 then
            print("Error: failed to load i3d file '" .. filename .. "'.");
        else
            id = clone(sharedI3D.nodeId, false, callOnCreate, addToPhysics);
        end;

        sharedI3D.refCount = sharedI3D.refCount + 1;
        return id;
    end;
end;

I3DManager.loadSharedI3DFile = Utils.overwrittenFunction(I3DManager.loadSharedI3DFile, loadSharedI3DFileFix);